<?php
/**
 * Package Cache
 *
 * description ...
 *
 * @package    Cache
 * @author
 * @copyright
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */

/**
 * Class Cache
 *
 * description ...
 *
 * @package    Cache
 * @version
 */
abstract class Cache {

  // -------------------------------------------------------------------------
  // ABSTRACT
  // -------------------------------------------------------------------------

  /**
   * Function set...
   *
   * @public
   * @param string $key
   * @param mixed $data
   * @return void
   */
  abstract public function set( $key, $data );

  /**
   * Function get...
   *
   * @public
   * @param string $key
   * @param int $timestamp
   * @return mixed Return NULL in case of not found, '' in case of empty found
   */
  abstract public function get( $key, $timestamp=0 );

  /**
   * Function remove...
   *
   * @public
   * @param string $key
   * @return void
   */
  abstract public function remove( $key );

  /**
   * Function clear...
   *
   * @public
   * @return void
   */
  abstract public function clear();

  // -------------------------------------------------------------------------
  // PUBLIC
  // -------------------------------------------------------------------------

  /**
   * Function Init...
   *
   * @public
   * @param array $Settings
   * @return void
   */
  public static function Init( $class, $Settings=array() ) {
    if (isset(self::$Instance))
      throw new CacheException(__CLASS__.' is still initialized!');
    $class = 'Cache_'.$class;
    self::$Instance = new $class($Settings);
  }

  /**
   * Function getInstance...
   *
   * @public
   * @return CacheI
   */
  public static function &getInstance() {
    if (!isset(self::$Instance))
      throw new CacheException(__CLASS__.' is not initialized, call Init() before!');
    return self::$Instance;
  } // function getInstance()

  /**
   * Function save...
   *
   * <code>
   * while (Cache::save($key, $data[, $timestamp])) {
   *   ...
   *   $data = ...;
   * }
   * echo $data;
   * </code>
   *
   * @public
   * @param string $key
   * @param mixed &$data
   * @param int $timestamp
   * @return void
   */
  public function save( $key, &$data, $timestamp=0 ) {
    if ($key == end($this->Stack)) {
      $this->set($key, $data);
      // done, remove id from stack
      array_pop($this->Stack);
      return FALSE;
    } elseif (in_array($key, $this->Stack)) {
      // $key is in stack, but NOT on top
      throw new CacheException('Cache stack problem: '.end($this->Stack).' not properly finished!');
    } else {
      $data = $this->get($key, $timestamp);
      if ($data !== NULL) {
        // Content found in cache, done
        return FALSE;
      } else {
        // not found yet, let's go
        $this->Stack[] = $key;
        return TRUE;
      }
    }
  } // function save()

  // -------------------------------------------------------------------------
  // PROTECTED
  // -------------------------------------------------------------------------

  /**
   * Class constructor
   *
   * @protected
   * @param array $Settings
   * @return void
   */
  protected function __construct( $Settings=array() ) {
    $this->Stack = array();
  } // function __construct()

  /**
   * Function map...
   *
   * @protected
   * @param string $key
   * @return string
   */
  protected function map( $key ) {
    return strtolower($key);
  } // function map()

  // -------------------------------------------------------------------------
  // PRIVATE
  // -------------------------------------------------------------------------

  /**
   *
   * @private
   * @var CacheI
   */
  private static $Instance = NULL;

  /**
   * @var array
   */
  private $Stack;

}

/**
 * Class CacheException
 *
 * description ...
 *
 * @package    Cache
 * @version
 */
class CacheException extends Exception {}