<?php
/**
 * Package Cache
 *
 * description ...
 *
 * @package    Cache
 * @author
 * @copyright
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */

/**
 * Class Cache_File
 *
 * description ...
 *
 * @package    Cache
 * @version
 */
class Cache_File extends Cache {

  // -------------------------------------------------------------------------
  // PUBLIC
  // -------------------------------------------------------------------------

  /**
   *
   * @public
   * @var string
   */
  public $CacheDir = '/tmp';

  /**
   *
   * @public
   * @var string
   */
  public $Token;

  /**
   * Function set...
   *
   * @public
   * @param string $key
   * @param mixed $data
   * @return mixed
   */
  public function set( $key, $data ) {
    file_put_contents($this->FileName($key), serialize($data));
  } // function set()

  /**
   * Function get...
   *
   * @public
   * @param string $key
   * @param int $timestamp
   * @return mixed|NULL
   */
  public function get( $key, $timestamp=0 ) {
    $file = $this->FileName($key);
    if (file_exists($file)) {
      if ($timestamp <= filemtime($file)) {
        return unserialize(file_get_contents($file));
      } else {
        unlink($file);
      }
    }
  } // function get()

  /**
   * Function remove...
   *
   * @public
   * @param string $key
   * @return void
   */
  public function remove( $key ) {
    $file = $this->FileName($key);
    file_exists($file) && unlink($file);
  } // function remove()

  /**
   * Function clear...
   *
   * @public
   * @return void
   */
  public function clear() {
    foreach (glob($this->CacheDir.DIRECTORY_SEPARATOR.$this->Token.'.*.cache') as $file) {
      unlink($file);
    }
  } // function clear()

  // -------------------------------------------------------------------------
  // PROTECTED
  // -------------------------------------------------------------------------

  /**
   * Class constructor
   *
   * @public
   * @param array $options
   * @return CacheI
   */
  protected function __construct( $Settings=array() ) {
    parent::__construct($Settings);

    $this->Token = !empty($Settings['Token']) ? $Settings['Token'] : md5(__FILE__);
    if (!empty($Settings['CacheDir'])) $this->CacheDir = $Settings['CacheDir'];
  }

  /**
   * Function FileName...
   *
   * @protected
   * @param string $key
   * @return string
   */
  protected function FileName( $key='' ) {
    if ($key) $key = '.'.md5(parent::map($key));
    return $this->CacheDir.DIRECTORY_SEPARATOR.$this->Token.$key.'.cache';
  } // function map()

  // -------------------------------------------------------------------------
  // PRIVATE
  // -------------------------------------------------------------------------

}