<?php
/**
 * Program initialization
 *
 * @package    es-f
 * @subpackage Core
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2007-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    1.1.2
 * @since      File available since Release 0.0.1
 */

defined('_ESF_OK') || die('No direct call allowed.');

// set error reporting
error_reporting(E_ALL ^ E_WARNING ^ E_NOTICE);

// no session auto start
ini_set('session.auto_start', 0);

// clear file cache
clearstatcache();

// force UTF-8
Header('Content-Type: text/html; charset=utf-8');

/**
 * P3P Policy (http://www.w3.org/P3P/)
 *
 * Source: http://www.phpopentracker.de/docs/en/installation.html
 *
 * The MS Internet Explorer 6, and maybe other browsers, too, requires a
 * Platform for Privacy Preferences (P3P) Project policy in order to accept
 * cookies.
 *
 * This includes session cookies that are used by PHP's session management
 * system, on which |es|f| relies. |es|f| cannot handle users using the MS
 * Internet Explorer 6 correctly if the site does not send such a policy to the
 * client.
 *
 * The code below shows a suitable P3P policy.
 */
Header('P3P: CP="NOI NID ADMa OUR IND UNI COM NAV"');

/**
 * Application users
 *
 * @global array $GLOBALS['users']
 */
$GLOBALS['users'] = array();

/**
 * Set new separator for namespaces in registry
 */
Registry::$NameSpaceSeparator = '.';

/**
 * Application configuration
 */
#Registry::set('esniper',     'esniper');
Registry::set('SuDo',        '');
#Registry::set('RunDir',      '.run');
Registry::set('Language',    'de');
Registry::set('Layout',      'default');
Registry::set('MenuStyle',   'image,text,image');
// since Version 2
Registry::set('Netmask',     '255.255.255.255');
// since Version 3
Registry::set('ebayTLD',     'de');
// since Version 4
Registry::set('Cache',       FALSE);  // future use
Registry::set('SessionName', ini_get('session.name'));
Registry::set('LogFile',     '');
Registry::set('ParseOrder',  'de,com,co_uk');
// since Version 5
#Registry::set('Module',      'index');
#Registry::set('bin_esniper',  'esniper');
Registry::set('bin_esniper',  '/usr/local/esniper/esniper');
Registry::set('StartModule',  'index');
Registry::set('TimeZone',     'GMT');
Registry::set('Locale',       'C');
Registry::set('RunDir',       BASEDIR.'/local/data');
//Registry::set('Develop',     FALSE);

/**
 * esniper configuration
 */
Esniper::set('seconds',     10);
Esniper::set('debug',       'no');
Esniper::set('logdir',      '');
Esniper::set('proxy',       '');
// since Version 2
Esniper::set('historyHost', '');
Esniper::set('prebidHost',  '');
Esniper::set('bidHost',     '');
Esniper::set('loginHost',   '');
Esniper::set('myeBayHost',  '');

/**
 * Global application data
 */
Registry::set('esf.contentonly', FALSE);

// use own session name
Registry::set('SessionName', 'ESFSESSID');
#Session::$NVL = '';

Exec::InitInstance(ESF_OS);

/**
 * Somme common URLs
 */
Registry::set('URL.CreateBugTrackerItem', 'http://sourceforge.net/tracker/?func=add&group_id=185222&atid=912405');
Registry::set('URL.es-f.Homepage', 'http://es-f.com');
Registry::set('URL.es-f.SourceForge', 'http://sourceforge.net/projects/es-f');


/**
 * Configure TemplateData System
 */
TplData::$NameSpaceSeparator = '.';
TplData::$KeysUppercase = TRUE;
TplData::$NVL = NULL;

/**
 * Configure I18n System
 */
Translation::$NameSpaceSeparator = '.';

// >> Debug
/** --------------------------------------------------------------------------
 * Helper functions
 */

/**
 * @ignore
 */
function _dbg( $var, $name='' ) {
  ob_start();
  $options = array( 'name' => $name );
  new dBug($var, $options);
  Messages::addInfo(ob_get_clean(), TRUE);
}

/**
 * @ignore
 */
function __dbg( $var, $name='' ) {
  $options = array( 'name' => $name );
  new dBug($var, $options);
}

/**
 * @ignore
 */
function _dump() {
  foreach (func_get_args() as $expression) {
    ob_start();
    var_dump($expression);
    Messages::addInfo('<pre>'.ob_get_clean().'</pre>', TRUE);
  }
}
// << Debug
