<?php
/**
 * Template compiler class
 *
 * @ingroup  Core
 * @version  3.0.0
 * @author   Knut Kohl <knutkohl@users.sourceforge.net>
 */
class Yuelo_Compiler {

  /**
   * Holds last compiler error
   */
  public $Error = '';

  /**
   * @return Single instance of compiler object
   */
  public static function getInstance() {
    if (!is_object(self::$Instance)) self::$Instance = new self;
    return self::$Instance;
  }

  /**
   * Compiles HTML template into PHP code
   *
   * @param Yuelo_Template $Yuelo_Template Calling template object
   * @param string &$page HTML to transform to PHP code
   * @return bool TRUE on success, FALSE on error with error message in $this->Error
   */
  public function Compile( &$page ) {

    $CfgControlBegin     = Yuelo::get('ControlBegin');
    $CfgControlEnd       = Yuelo::get('ControlEnd');
    $CfgVariableBegin    = Yuelo::get('VariableBegin');
    $CfgVariableEnd      = Yuelo::get('VariableEnd');
    $CfgTranslationBegin = Yuelo::get('TranslationBegin');
    $CfgTranslationEnd   = Yuelo::get('TranslationEnd');
    $CfgCommentBegin     = Yuelo::get('CommentBegin');
    $CfgCommentEnd       = Yuelo::get('CommentEnd');

    $this->Externals = Yuelo::get('_Externals');
    $this->ExtensionTagged = array( 'E' => array(), 'F' => array() );

    $TplHeader = $TplFooter = '';
    $TplInfo = array();

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Macros
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $macros = $codes = array();
    foreach (Yuelo::get('_Macros') as $macro=>$code) {
      $macros[] = '@'.$macro.'@';
      $codes[]  = $code;
    }
    $page = str_replace($macros, $codes, $page);
    unset($macros, $codes, $macro, $code);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  PHP code allowed?
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (!Yuelo::get('AllowPHP')) {
      $page = str_ireplace('<?php', '&lt;?php', $page);
      $page = str_ireplace('?'.'>', '?&gt;', $page);
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  extract template informations
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = sprintf('~%1$s.*?(^[\s*]*@\w+.*?)%2$s~ms',
                     $CfgCommentBegin, $CfgCommentEnd);
    if (preg_match_all($regex, $page, $args)) {
      foreach ($args[1] as $tag) {
        if (preg_match_all('~^[\s*]*@(\w+)\s+(.*?)$~m', $tag, $var)) {
          foreach ($var[1] as $id => $info) {
            $TplInfo[] = sprintf(' * %s: %s', ucwords($info), $var[2][$id]);
          }
        }
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  remove template comment blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = sprintf('~%1$s.*?%2$s~s', $CfgCommentBegin, $CfgCommentEnd);
    $page = preg_replace($regex, '', $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  extract block definitions
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = sprintf(
    //                   1        2
      '~%1$sBEGIN BLOCK (\w+)%2$s(.*)%1$sEND BLOCK \\1%2$s~s',
      preg_quote($CfgControlBegin, '~'), preg_quote($CfgControlEnd, '~')
    );
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        // remove block definition
        $page = str_replace($arg[0], '', $page);
        // replace block place holder
        $block = sprintf('%sBLOCK %s%s', $CfgControlBegin, $arg[1], $CfgControlEnd);
        $page = str_replace($block, $arg[2], $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Check for missing block definitions, always mark as error!
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //        1
      'BLOCK (\w+).*?', $CfgControlBegin, $CfgControlEnd);
      if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        $code = sprintf('<span style="color:red">Missing block definition [%s]!</span>', $arg[1]);
        $page = str_replace($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  mask masked variable start/end tags
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $page = str_replace('\\'.$CfgVariableBegin,    "\x01" ,$page);
    $page = str_replace('\\'.$CfgVariableEnd,      "\x02" ,$page);
    $page = str_replace('\\'.$CfgTranslationBegin, "\x03" ,$page);
    $page = str_replace('\\'.$CfgTranslationEnd,   "\x04" ,$page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Replace Internationalizations {...[[...]]...} INSIDE extension calls
    //  with I18N.... for processing just as array scalars
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = sprintf('~(%s[^}]*?)%s(%s)%s([^}]*?%s)~',
                     preg_quote($CfgVariableBegin, '~'),
                     preg_quote($CfgTranslationBegin, '~'),
                     $this->VarRegexFilter,
                     preg_quote($CfgTranslationEnd, '~'),
                     preg_quote($CfgVariableEnd, '~'));
    while (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        $code = sprintf('%sI18N.%s%s', $arg[1], $arg[2], $arg[3]);
        $page = str_replace($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Replace Internationalizations [[...]]
    //  with {I18N....} for processing just as array scalars
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex('('.$this->VarRegexFilter.')',
                               $CfgTranslationBegin, $CfgTranslationEnd);
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        $code = sprintf('%sI18N.%s%s', $CfgVariableBegin, $arg[1], $CfgVariableEnd);
        $page = str_replace($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  change assign constants to a variable
    //  {"0"[,x] > var} to {:"0"[,x] > var}
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $page = str_replace($CfgVariableBegin.'"', $CfgVariableBegin.':"', $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  change multiple var assigns to a variable to a "concatenate"
    //  {a[,b] > var} to {:a[,b] > var}
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex('([^:\s>}]+\s+>)', $CfgVariableBegin, '');
    $page = preg_replace($regex, $CfgVariableBegin.':$1', $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  'BEGIN - END' Blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //                1
      sprintf('BEGIN (%s)', $this->VarRegex), $CfgControlBegin, $CfgControlEnd);
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        list($parent, $block, ) = $this->VarName($arg[1]);
        $loop = '$YueloLoop' . rand(10000, 99999);
        // BEGIN ...
        $code = sprintf($this->LoopBeginCode, $parent, $block, $loop);
        $this->ReplaceTag($arg[0], $code, $page, FALSE, TRUE);
        // END ...
        $arg[0] = str_replace('BEGIN', 'END', $arg[0]);
        $code = sprintf($this->LoopEndCode, $loop);
        $this->ReplaceTag($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  'IF nnn="mmm"'    or  'IF nnn=mmm'  or
    //  'IF nnn = "mmm"'  or  'IF nnn = mmm' Blocks
    //  'ELSEIF nnn="mmm"'    or  'ELSEIF nnn=mmm'  or
    //  'ELSEIF nnn = "mmm"'  or  'ELSEIF nnn = mmm' Blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //          1         2   3         4
      sprintf('(ELSE)?IF (%s)([ !=<>]+)([^ ]+)', $this->VarRegexFilter),
      $CfgControlBegin, $CfgControlEnd
    );
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        list($parent, $block, $filters) = $this->VarName($arg[2]);
        $cmp = $this->Comparison($arg[3]);
        $val = $arg[4];
        $const = preg_match('~^"([^"]+)"$~', $val, $constval);
        if ($const and is_numeric($val[1])) $val = $constval[1];
        if (!$const and !empty($val))
          list($parent1, $block1, $filters1) = $this->VarName($val);
        $code1 = '@' . $this->VarCode($parent, $block);
        if (!$this->addFilters($code1, $filters)) return FALSE;
        if ($cmp.$val == '==""') {
          $code2 = '\'\'';
        } elseif ($const) {
          $code2 = $val;
        } else {
          $code2 = '@' . $this->VarCode($parent1, $block1);
          if (!$this->addFilters($code2, $filters1)) return FALSE;
        }
        $code = sprintf('%sif (%s %s %s):', $arg[1], $code1, $cmp, $code2);
        $this->ReplaceTag($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  'IF nnn' Blocks
    //  'ELSEIF nnn' Blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //          1         2      3
      sprintf('(ELSE)?IF (!)?\s*(%s)', $this->VarRegexFilter),
      $CfgControlBegin, $CfgControlEnd
    );
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        list($parent, $block, $filters) = $this->VarName($arg[3]);
        $code = '@' . $this->VarCode($parent, $block);
        if (!$this->addFilters($code, $filters)) return FALSE;
        $code = sprintf('%sif (%s%s):', $arg[1], $arg[2], $code);
        $this->ReplaceTag($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  'IF {extension:variable} = mmm' Blocks
    //  'ELSEIF {extension:variable} = mmm' Blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //  1          2      3       4         5
      '(ELSE)?IF {(\w*?):([^}]*)}([ !=<>]*)([^ ]*)',
      $CfgControlBegin, $CfgControlEnd
    );
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        $cmp  = $this->Comparison($arg[4]);
        if ($res = $this->AnalyseExtensionCall($arg[2], $arg[3])) {
          // need only the generated code
          list(, $code) = $res;
        } else {
          return FALSE;
        }
        $code = sprintf('%sif (%s %s %s):', $arg[1], $code, $cmp, $arg[5]);
        $this->ReplaceTag($arg[0], $code, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  ELSE, ENDIF Blocks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $tag = sprintf('%sELSE%s', $CfgControlBegin, $CfgControlEnd);
    $this->ReplaceTag($tag, 'else:', $page);

    $regex = $this->BuildRegex('ENDIF.*?', $CfgControlBegin, $CfgControlEnd);
    $this->ReplaceTag($regex, 'endif;', $page, TRUE);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  remove inplace  comments in {.../*...*/...}
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = '~('.$CfgVariableBegin.'.*?)\s*/\*.*?\*/\s*(.*?'.$CfgVariableEnd.')~';
    $page = preg_replace($regex, '$1$2', $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  INCLUDE statements
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //          1
      'INCLUDE ([^\s]+)', $CfgControlBegin, $CfgControlEnd);
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      // prepare subpage object
      $IncTpl = '$YueloInc' . rand(10000, 99999);
      // replace/insert code to parse subtemplates
      $Includes = array();
      $IncCode = '%1$s->setData($YueloData); %1$s->Output(\'%2$s\');';
      foreach ($args as $arg) {
        $Includes[] = ' * - '.$arg[1];
        $code = sprintf($IncCode, $IncTpl, $arg[1]);
        $this->ReplaceTag($arg[0], $code, $page);
      }
      $TplHeader .= '/**' . "\n"
                   .' * Includes:' . "\n"
                   .implode("\n", $Includes) . "\n"
                   .' */' . "\n"
                   .$IncTpl.' = clone($this);' . "\n";
      $TplFooter .= 'unset('.$IncTpl.');';
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Hide HTML comments from compiling or remove them
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (preg_match_all('~<!--.*?-->~s', $page, $comments)) {
      if (Yuelo::get('Verbose') & Yuelo::VERBOSE_COMMENTS) {
        foreach ($comments[0] as $id => $cmt) $comments[1][$id] = md5($cmt);
        $page = str_replace($comments[0], $comments[1], $page);
      } else {
        $page = str_replace($comments[0], '', $page);
        unset($comments);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Replace Scalars
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //          1
      sprintf('(%1$s(?: > %1$s)?)', $this->VarRegexFilter),
      $CfgVariableBegin, $CfgVariableEnd
    );
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        // Determine Command (echo / $obj[n]=)
        list($cmd, $tag) = $this->CmdName($arg[1]);
        list($block, $skalar, $filters) = $this->VarName($tag);
        $code = $this->VarCode($block, $skalar);
        if ($cmd = 'echo' AND
            Yuelo::get('Verbose') & Yuelo::VERBOSE_MARKMISSING) {
          $code = sprintf('(isset(%1$s) ? %1$s : \'<span %2$s>\{%3$s\}</span>\')',
                          $code, $this->MissingVarMark, $arg[1]);
        } else {
          // suppress notices
          $code = '@'.$code;
        }
        if (!$this->addFilters($code, $filters)) return FALSE;
        $this->ReplaceTag($arg[0], $cmd.' '.$code.';', $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Include Extensions
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $regex = $this->BuildRegex(
    //  1      2
      '(\w*?):([^}]*)', $CfgVariableBegin, $CfgVariableEnd);
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      foreach ($args as $arg) {
        if (!$res = $this->AnalyseExtensionCall($arg[1], $arg[2])) return FALSE;
        list($cmd, $code, $strictcode) = $res;
        $this->ReplaceTag($arg[0], $cmd.' '.$code.';'.$strictcode, $page);
      }
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  Unmask masked data
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $page = str_replace("\x01", '\\'.$CfgVariableBegin,    $page);
    $page = str_replace("\x02", '\\'.$CfgVariableEnd,      $page);
    $page = str_replace("\x03" ,'\\'.$CfgTranslationBegin, $page);
    $page = str_replace("\x04" ,'\\'.$CfgTranslationEnd,   $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  get back masked comments
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (!empty($comments[1]))
      $page = str_replace($comments[1], $comments[0], $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  unmask masked brackets
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $page = str_replace("\x01", $CfgVariableBegin, $page);
    $page = str_replace("\x02", $CfgVariableEnd, $page);

    $page = str_replace('\\'.$CfgVariableBegin, $CfgVariableBegin, $page);
    $page = str_replace('\\'.$CfgVariableEnd, $CfgVariableEnd, $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  compress a little bit
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    $page = preg_replace('~\s*\?'.'>\s+<\?php\s*echo\s*~si', ' echo \' \',', $page);
    $page = preg_replace('~\s*\?'.'>\s*<\?php\s*echo\s*~si', ' echo ', $page);
    $page = preg_replace('~\s*\?'.'>\s+<\?php\s*~si', ' ', $page);
    $page = preg_replace('~\s*\?'.'>\s*<\?php\s*~si', '', $page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  is this page cachable, contains no PHP code
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (!preg_match('~<\?php~i', $page)) $page .= Yuelo::CACHETAG;

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    //  add include header & footer
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (!Yuelo::get('AutoLoad') AND
        (!empty($this->ExtensionTagged['E']) OR
         !empty($this->ExtensionTagged['F']))) {
      $code = '<?php' . "\n/**\n * Extensions / Filters\n */\n";
      $path = dirname(__FILE__);
      foreach ($this->ExtensionTagged['E'] as $file)
        $code .= 'require_once \'' . $file . '\';'. "\n";
      foreach ($this->ExtensionTagged['F'] as $file)
        $code .= 'require_once \'' . $file . '\';'. "\n";
      $page = $code . '?'.'>' . "\n" . $page;
    }

    if (!empty($TplHeader))
      $page = '<?php'    . "\n"
            . $TplHeader . "\n"
            . '?'.'>'    . "\n"
            . $page;
    if (!empty($TplInfo))
      $page = '<?php'                 . "\n"
            . '/**'                   . "\n"
            . implode("\n", $TplInfo) . "\n"
            . ' */'                   . "\n"
            . '?'.'>'                 . "\n"
            . $page;
    if (!empty($TplFooter))
      $page .=              "\n"
             . '<?php '   . "\n"
             . $TplFooter . "\n"
             . ' ?'.'>';

    // remove some line breaks...
    $page = preg_replace('~^\s+$~m', '', $page);   // mostly empty line
    $page = preg_replace('~\n\n+~', "\n", $page);  // multiple line breaks
    $page = preg_replace('~\?'.'>\s*$~', '', $page);
    $page = trim($page);

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // syntax checks
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // - orphan loop ENDs
    $regex = $this->BuildRegex(
    //              1
      sprintf('END (%s)', $this->VarRegex), $CfgControlBegin, $CfgControlEnd);
    if (preg_match_all($regex, $page, $args, PREG_SET_ORDER)) {
      $errors = array();
      foreach ($args as $arg)
        $errors[] = sprintf('Missing BEGIN for block "%s"', $arg[1]);
      $this->Error = implode('; ', $errors);
      return FALSE;
    }

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // Success
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    return TRUE;
  }

  //---------------------------------------------------------------------------
  // PRIVATE
  //---------------------------------------------------------------------------

  /**
   * Instance for Singleton
   */
  private static $Instance = NULL;

  /**
   * Complete mask for variable names
   */
  private $VarRegex;

  /**
   * Complete mask for variable names inclucive filter functions
   */
  private $VarRegexFilter;

  /**
   * Style or class definition
   */
  private $MissingVarMark;

  /**
   * Extension / Filters directories
   */
  private $ExtensionDirs;

  /**
   * List of used Yuelo extensions by template to build include header
   */
  private $ExtensionTagged;

  /**
   * List of registered Yuelo extensions
   */
  private $Externals;

  /**
   * PHP code for loop starts
   */
  private $LoopBeginCode = '
if (isset($%1$s[\'%2$s\']) AND !empty($%1$s[\'%2$s\'])):
  $YueloStack[$YueloStackCnt++] =& $YueloData;
  %3$sCnt = 0;  %3$sC = count($%1$s[\'%2$s\']);
  foreach ($%1$s[\'%2$s\'] as %3$sID => %3$s):
    %3$sCnt++;
    // force %3$s is an array
    if (!is_array(%3$s)) %3$s = array(\'%2$s\' => %3$s);
    // force array keys of %3$s are upper case
    %3$s = array_change_key_case(%3$s, CASE_UPPER);
    %3$s[\'$ROWID\'] = %3$sID;
    %3$s[\'$ROWBIT\'] = %3$sCnt %% 2;
    %3$s[\'$ROWFIRST\'] = (%3$sCnt == 1);
    %3$s[\'$ROWLAST\'] = (%3$sCnt == %3$sC);
    %3$s[\'TRUE\'] = TRUE;
    %3$s[\'FALSE\'] = FALSE;
    $YueloData =& %3$s;
    $YueloData = array_merge($YueloStack[0], $YueloData);
';

  /**
   * PHP code for loop ends
   */
  private $LoopEndCode = '
  endforeach;
  $YueloData =& $YueloStack[--$YueloStackCnt];
  unset(%1$s, %1$sId, %1$sC, %1$sCnt);
endif;
';

  /**
   * Initializes some internal variables
   *
   * @return void
   */
  private function __construct() {
    /*
     * Mask for internal variable names:
     * - loop variables: \$ROW\w+
     * - directory variables: \$\w+DIR
     */
    $this->VarRegex = sprintf('(?:%s|\$ROW\w+|_*\$\w+DIR)', Yuelo::get('VarRegexExternal'));
    $this->VarRegexFilter = $this->VarRegex.'[a-zA-Z0-9|_]*';

    $style = Yuelo::get('MissingVarStyle');
    $this->MissingVarMark = strstr($style, ':')
                            // only CSS code can/must contain a ":"...
                          ? sprintf('style="%s"', $style)
                            // ...otherwise CSS class name
                          : sprintf('class="%s"', $style);

    // put into array for simple reference by type
    $path = dirname(__FILE__);
    $this->ExtensionDirs = array(
      'E' => $path.'/extension',
      'F' => $path.'/filter',
    );
  }

  /**
   * Build well formed reg. expressions
   *
   * @param $regex Reg. expression
   * @param $begin Prefix
   * @param $end Suffix
   * @param $modifier Pattern modifier
   * @return string
   */
  private function BuildRegex( $regex, $begin='', $end='', $modifier='' ) {
    return '~' . preg_quote($begin, '~') . $regex
               . preg_quote($end, '~') . '~' . $modifier;
  }

  /**
   * Analyse required extensions
   *
   * @param string $type Extension or filter
   * @param string $func Extension function
   * @return string|bool
   */
  private function FindExtension( $type, $func ) {
    // still tagged?
    if (isset($this->ExtensionTagged[$type][$func])) return TRUE;

    // User defined extension/filter OVERWRITE default extensions/filters!
    if (isset($this->Externals[$type][$func])) {
      $this->ExtensionTagged[$type][$func] = $this->Externals[$type][$func];
      return TRUE;
    }

    $file = sprintf('%s/%s.class.php', $this->ExtensionDirs[$type], $func);
    if (file_exists($file)) {
      // Default extension/filter
      $this->ExtensionTagged[$type][$func] = $file;
      return TRUE;
    }

    $this->Error = 'Missing Yuelo Extension/Filter ['.$type.']: '.$func.' ('.$file.')';
    return FALSE;
  }

  /**
   * Add filter functions
   *
   * @param string &$code Code to include in filter functions
   * @param array $filters Filter functions
   * @return bool TRUE on success, FALSE on error
   */
  private function addFilters( &$code, $filters ) {
    foreach ($filters as $filter) {
      if ($this->FindExtension('F', $filter) === FALSE) return FALSE;
      $code = sprintf('Yuelo_Filter_%s::Process(%s)', $filter, $code);
    }
    return TRUE;
  }

  /**
   * Replace placeholder with php code
   *
   * @param string $tag placeholder
   * @param string $code php code
   * @param string $page page HTML
   * @param boolean $regex replace using reg. expression
   * @param boolean $NL replace surrounded by newlines
   * @return void
   */
  private function ReplaceTag( $tag, $code, &$page, $regex=FALSE, $NL=FALSE ) {
    $code = '<?php '.$code.' ?'.'>';
    if ($NL) $code = "\n".$code."\n";
    $page = !$regex ? str_replace($tag, $code, $page)
                    : preg_replace($tag, $code, $page);
  }

  /**
   * Analyse extension code call
   *
   * @param string $function Extension function
   * @param string $params Function parameters
   * @return array|FALSE
   */
  private function AnalyseExtensionCall( $function, $params ) {
    $function = trim($function);

    //  Determin Command (echo / $obj[n]=)
    list($cmd, $tag) = $this->CmdName($params);

    // check existence of extension
    if (!empty($function) AND $function <> 'concat' AND
        $this->FindExtension('E', $function) === FALSE)
      return FALSE;

    // mask masked quote
    $tag = str_replace('\"', "\x06", $tag);

    preg_match_all('~("[^"]*"|[^,]+)?,~', $tag.',', $tags);
    $tags = $tags[1];

    $tagarr = array();
    $strictcode = '';
    foreach ($tags as $tag) {
      $tag = trim($tag);
      if (empty($tag)) {
        $tagarr[] = 'NULL';
      } elseif (preg_match('~^([\d.]+)$~', $tag, $args)) {
        // numeric constant
        $tagarr[] = $args[1];
      } elseif (preg_match('~^"(.*)"$~', $tag, $args)) {
        // constants
        $tagarr[] = '\''.str_replace('\'', '\\\'', $args[1]).'\'';
      } else {
        // variables
        list($block, $skalar, $filters) = $this->VarName($tag);

        $_var = $this->VarCode($block, $skalar);

        if (Yuelo::get('Verbose') & Yuelo::VERBOSE_MARKMISSING) {
          if (!$this->addFilters($_var, $filters)) {
            return FALSE;
          }
          if ($skalar !== '') {
            $_var = sprintf('(isset($%s[\'%s\']) ? %s : NULL)', $block, $skalar, $_var);
            if ($function != 'nvl') {
              // if function is not {nvl:...},
              // check that the function is called with existing values
              $span = sprintf('<span %s>%s%s%s</span>', $this->MissingVarMark, "\x01", $tag, "\x02");
              $strictcode .= sprintf(' if (!isset($%s[\'%s\'])): echo \'%s\'; endif;',
                                     $block, $skalar, $span);
            }
          }
        } else {
          // suppress notices
          $_var = '@'.$_var;
          if (!$this->addFilters($_var, $filters)) {
            return FALSE;
          }
        }
        $tagarr[] = $_var;
      }
    }

    if (empty($function) OR $function == 'concat') {
      $code = implode('.',$tagarr);
      $code = preg_replace('~\'\.\'~', '', $code);
    } else {
      $code = sprintf('Yuelo_Extension_%s::Process(%s)', $function, implode(',',$tagarr));
    }
    $code = str_replace("\x06", '"', $code);

    return array($cmd, $code, $strictcode);
  }

  /**
   * Splits Template-Style Variable Names into an Array-Name/Key-Name Components
   *
   * @code
   * {example}                 : array("YueloData",                    "example") -> $YueloData['example']
   * {example.value}           : array("YueloData['example']",         "value")   -> $YueloData['example']['value']
   * {example.0.value}         : array("YueloData['example'][0]",      "value")   -> $YueloData['example'][0]['value']
   * {_top.example}            : array("YueloStack[0]",                "example") -> $YueloStack[0]['example']
   * {__example}               : array("YueloStack[0]",                "example") -> $YueloStack[0]['example']
   * {_parent.example}         : array("YueloStack[$YueloStackCnt-1]", "example") -> $YueloStack[$YueloStackCnt-1]['example']
   * {_parent._parent.example} : array("YueloStack[$YueloStackCnt-2]", "example") -> $YueloStack[$YueloStackCnt-2]['example']
   * @endcode
   *
   * @param string $tag Variale Name used in Template
   * @return array Array(Name, Key Name, Filters)
  */
  private function VarName( $tag ) {
    $filters = explode('|', trim($tag));
    $tag = trim(array_shift($filters));

    // Force variables uppercase
    if (Yuelo::get('VarNamesUppercase')) $tag = strtoupper($tag);

    // analyse parent levels
    $ParentLevel = 0;
    while (preg_match('~^_PARENT\.?(.*)$~i', $tag, $args)) {
      $ParentLevel++;
      $tag = $args[1];
    }

    if ($ParentLevel) {
      $obj = 'YueloStack[$YueloStackCnt-'.$ParentLevel.']';
    } elseif (preg_match('~^(CONST|I18N)\.?(.*)$~i', $tag, $args)) {
      $obj = 'Yuelo'.strtoupper($args[1]);
      $tag = $args[2];
    } elseif (preg_match('~^(?:__|_TOP\.)(.*)$~i', $tag, $args)) {
      // tags beginning with "__" are interpreted as short form of _TOP level
      $obj = 'YueloStack[0]';
      $tag = $args[1];
    } else {
      $obj = 'YueloData';
    }

    while (strpos($tag,'.') !== FALSE) {
      list($parent, $tag) = explode('.', $tag, 2);
      $obj .= is_numeric($parent) ? "[$parent]" : "['$parent']";
    }
    return array($obj, $tag, $filters);
  }

  /**
   * Build variable name from block and skalar
   *
   * Return block if skalar is empty
   *
   * Example:
   * @code
   * VarCode('block', 'skalar')  ==>  $block['skalar']
   * VarCode('block', '')        ==>  $block
   * @endcode
   *
   * @param string $block Block array
   * @param string $skalar Variable inside $block
   * @return string
   */
  private function VarCode( $block, $skalar ) {
    return ($skalar == '') ? '$'.$block : '$'.$block.'[\''.$skalar.'\']';
  }

  /**
   * Determine Template Command from Variable Name
   *
   * @code
   * {variable}             :  array( "echo",                   "variable" )  ->  echo $YueloData['variable']
   * {variable > new_name}  :  array( "YueloData['new_name']=", "variable" )  ->  $YueloData['new_name'] = $YueloData['variable']
   * @endcode
   *
   * @param string $tag Variable Name used in Template
   * @return array  Array( Command, Variable )
   */
  private function CmdName( $tag ) {
    $tag = trim($tag);
    $regex = sprintf('~^(.+) > (%s)$~', $this->VarRegex);
    if (preg_match($regex, $tag, $tagvar)) {
      $tag = $tagvar[1];
      list($newblock, $newskalar) = $this->VarName($tagvar[2]);
      $cmd = sprintf('$%s[\'%s\'] = ', trim($newblock), trim($newskalar));
    } else {
      $cmd = 'echo';
    }
    return array($cmd, trim($tag));
  }

  /**
   * Normalize the comparison statement
   *
   * @param string &$cmp
   * @return string
   */
  private function Comparison( $cmp ) {
    switch (trim($cmp)) {
      case '=' :
      case '==':  return '===';
      case '!=':
      case '<>':  return '!==';
      default:    return $cmp;
    }
  }

  /**
   * Can't clone a Singleton
   */
  private function __clone() {}

}