<?php
/**
 * serve.php - File serving script
 *
 * @package    serve
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2010 Knut Kohl
 * @version    1.0.0
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */

/**
 * Manipulate image URLs to new relative file location
 *
 * @package    serve
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2010 Knut Kohl
 * @version    1.0.0
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 *
 * @param string $content CSS to compress
 * @return string
 */
function css_pre_serve( $content, $file ) {
  $path = dirname($file).'/';
  if (preg_match_all('~(url\s*\()(.*?)(\))~i', $content, $args, PREG_SET_ORDER)) {
    foreach ($args as $arg)
      $content = str_replace($arg[0], $arg[1].$path.$arg[2].$arg[3], $content);
  }
  return $content;
}

/**
 * Compress CSS
 *
 * @package    serve
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2010 Knut Kohl
 * @version    1.0.0
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 *
 * @param string $content CSS to compress
 * @return string
 */
function css_post_serve( $content, $files, $pretty ) {

  $s1 = strlen($content);

  if (!$pretty) {
    // remove comments
    $content = preg_replace('~/\*.*?\*/~s', '', $content);

    // compress
    $content = preg_replace('~[\r\n]*\s*([,:;{}])\s*[\r\n]*~', '$1', $content);
    $content = preg_replace('~;\s*\}~', '}', $content);

    // compress whitespaces
    $content = preg_replace('~\s+~', ' ', $content);
    $content = trim($content);
  }

  // reduce multiple charsets
  $regex = '~@charset[\s\'"]+(.*?)[\'";]+~i';
  if (preg_match($regex, $content, $charset))
    $content = '@charset "' . $charset[1] . '";' . preg_replace($regex, '', $content);

  $s2 = strlen($content);

  return "/* " . implode("\n   ", $files) . "\n"
       . sprintf('   Compressed %d to %d Bytes (%.1f%%) */'."\n", $s1, $s2, $s2*100/$s1)
       . $content;
}
