<?php

$GLOBALS['DEBUGSTACK_ADD_VERSIONS'] = TRUE;

// Configurations
Loader::Load(APPDIR.'/init.php');
Loader::Load(LOCALDIR.'/custom/init.php', TRUE, FALSE);

// include functions
Loader::Load(APPDIR.'/functions.php');

// >> Debug
$DbgFile = TEMPDIR.'/'.APPID.'.debug';
if (isset($_GET['DEBUG'])) {
  touch($DbgFile);
} elseif (isset($_GET['TRACE'])) {
  $_TRACE = TEMPDIR.'/trace-'.ESF_VERSION.'-'.date('Ymd-Hi').'.csv';
  File::write($DbgFile, $_TRACE);
} elseif (isset($_GET['STOP'])) {
  @unlink($DbgFile);
  Messages::addInfo('Debug off');
}

$_DEBUG = file_exists($DbgFile);
$_TRACE = $_DEBUG ? file_get_contents($DbgFile) : FALSE;
#DebugStack::$TimeUnit = DebugStack::MICROSECONDS;
DebugStack::Active($_DEBUG);
if ($_TRACE) {
  Messages::addInfo('<a href="?STOP">[Stop]</a> Debug trace is active: '.$_TRACE, TRUE);
#  DebugStack::$TraceFile = $_TRACE;
} elseif ($_DEBUG)
  Messages::addInfo('<a href="?STOP">[Stop]</a> Debug active!', TRUE);
unset($DbgFile);
// << Debug

$options = array('CacheDir' => TEMPDIR);
Cache::Init(Registry::get('CacheClass', 'File_Combined'), $options);

if (Registry::get('ClearCache')) Cache::getInstance()->clear();

$xml = new XML_Array_Configuration(Cache::getInstance());
$cfg = $xml->ParseXMLFile(LOCALDIR.'/config/config.xml');
if (!$cfg) die($xml->Error);

foreach ($cfg['users'] as $user) esf_User::set($user['name'], $user['auth']);
unset($cfg['users']);

foreach ($cfg['esniper'] as $key => $val) Esniper::set($key, $val);
unset($cfg['esniper']);

// Set all other into registry
Registry::set($cfg);
unset($xml, $cfg);

if (Registry::get('CfgVersion') < ESF_CONFIG_VERSION) {
  Cache::getInstance()->clear();
  Header('Location: setup/index.php?msg='
        .urlencode('Need reconfiguration because of configuration changes!'));
}
if (count(esf_User::getAll()) == 0) {
  Cache::getInstance()->clear();
  Header('Location: setup/index.php?msg='
        .urlencode('At least one user account have to be defined!'));
}

esf_Extensions::Init();

// load here to allow overwrite in local/config.php
Loader::Load(APPDIR.'/ebay.php');

// include additional configuration, mostly for development
Core::ReadConfigs('local');

ErrorHandler::register(Registry::get('ErrorHandler', 'default'));
// >> Debug
DebugStack::Register();
// << Debug

// since PHP 5.1.0
if (function_exists('date_default_timezone_set'))
  // http://php.net/manual/en/timezones.php
  date_default_timezone_set(Registry::get('TimeZone'));

$p = dirname($_SERVER['PHP_SELF']);
if (substr($p,-1)!= '/') $p .= '/';
define('BASEHTML', sprintf('%s://%s%s', ServerProtocol(), $_SERVER['HTTP_HOST'], $p));
unset($p);

TplData::setConstant('BASEHTML', BASEHTML);

Loader::Load('module/modules.php');
Loader::Load('plugin/plugins.php');

$Exec = Exec::getInstance();

// Load core and custom Exec commands
$Exec->setCommandsFromXMLFile(sprintf('%s/exec.%s.xml', APPDIR, ESF_OS));
$Exec->setCommandsFromXMLFile(BASEDIR.'/local/custom/exec.xml', FALSE);

if (IniFile::Parse(APPDIR.'/language/languages.ini')) {
  $esf_Languages = IniFile::$Data;
} else {
  Messages::addError(IniFile::$Error);
  $esf_Languages = array('en' => 'English');
}

/// DebugStack::StartTimer('LoadPlugins', 'Load plugins');

Core::ReadConfigs(esf_Extensions::MODULE);

Core::IncludeSpecial(esf_Extensions::MODULE, 'plugin.class', TRUE);

// include all plugin configs
################################
Core::IncludeSpecial(esf_Extensions::PLUGIN, 'config');
Core::ReadConfigs(esf_Extensions::PLUGIN);

Event::ProcessInform('PluginConfigsLoaded');

Core::IncludeSpecial(esf_Extensions::PLUGIN, 'plugin.class');

Event::ProcessInform('PluginsLoaded');
Event::ProcessInform('ModuleConfigsLoaded');

/// DebugStack::StopTimer('LoadPlugins');

################################
Core::StartSession();

Event::ProcessInform('SessionStarted');

if (PluginEnabled('Validate')) {
  DefineValidator('module', 'Regex', array('pattern'=>'\w*'));
  DefineValidator('action', 'Regex', array('pattern'=>'\w*'));
}

/// DebugStack::StartTimer('AnalyseRequestParams', 'Analyse request parameters');

// Emulate register_globals off
unregister_GLOBALS();

//_dbg($_COOKIE, '$_COOKIE');
//_dbg($_SERVER, '$_SERVER');
//_dbg($_SESSION);
//_dbg($_REQUEST);

Core::StripSlashes($_REQUEST);

/// DebugStack::Debug('$_REQUEST : '.print_r($_REQUEST, TRUE));

if (strtoupper($_SERVER['REQUEST_METHOD']) == 'GET') {
  /// DebugStack::Debug('$_GET before: '.print_r($_GET, TRUE));
  Core::StripSlashes($_GET);
  Event::Process('UrlUnRewrite', $_GET);
  Event::Process('AnalyseRequest', $_GET);
  /// DebugStack::Debug('$_GET after analyse: '.print_r($_GET, TRUE));
  $_POST = array();
} else {
  /// DebugStack::Debug('$_POST : '.print_r($_POST, TRUE));
  Core::StripSlashes($_POST);
  Event::Process('UrlUnRewrite', $_POST);
  Event::Process('AnalyseRequest', $_POST);
  /// DebugStack::Debug('$_POST after analyse: '.print_r($_POST, TRUE));
  $_GET = array();
}
Event::Process('UrlUnRewrite', $_REQUEST);
Event::Process('AnalyseRequest', $_REQUEST);

// analyse request
Registry::set('esf.Module',      checkR('module',      Registry::get('StartModule')));
Registry::set('esf.Action',      checkR('action',      'index'));
Registry::set('esf.contentonly', checkR('contentonly', FALSE));

/// DebugStack::StopTimer('AnalyseRequestParams');

// ----------------------------------------------------------------------------
// initialize application
// ----------------------------------------------------------------------------
Event::ProcessInform('Start');

Session::checkRequest('language', Registry::get('language'));

// only initiate empty array data, empty strings not required
TplData::set('HtmlHeader.JS', array());
TplData::set('HtmlHeader.CSS', array());
TplData::set('HtmlHeader.Script', array());

// >> Debug
if ($_DEBUG) {
  TplData::add('HtmlHeader.JS',  'html/serve.php?'.rawurlencode('../application/classes/debugstack.js'));
  TplData::add('HtmlHeader.CSS', 'html/serve.php?'.rawurlencode('../application/classes/debugstack.css'));
}
// << Debug

$esf_sellers = array();

/// DebugStack::StartTimer('CoreLangLoad', 'Load core languages');

if (Registry::get('EnglishAsDefault') AND $lang != 'en') {
  // load as default english texts and config
  Core::IncludeSpecial(NULL, array(APPDIR.'/language/en', APPDIR.'/language/config/en'));
}

// include translation
$lang = Session::get('language');
if (file_exists(APPDIR.'/language/'.$lang.'.php')) {
  // load as default english texts and config
  Core::IncludeSpecial(NULL, array(APPDIR.'/language/'.$lang, APPDIR.'/language/config/'.$lang));
} else {
  Messages::addError(sprintf('Unknown language [%s]! Fallback to english!', $lang));
  Session::set('language', 'en');
}
unset($lang);

if ($locale = Session::get('locale')) {
  setlocale(LC_ALL, $locale);
} else {
  if (!setlocale(LC_ALL, Registry::get('locale'))) {
    Messages::addError(sprintf('Locale <tt>['.Registry::get('locale').']</tt> not found on your system! '
                              .'Please <a href="setup/">reconfigure</a> your system and select a correct locale!'),
                       TRUE);
    Messages::addInfo(sprintf('Fall back for now to locale <tt>[%s]</tt>.', setlocale(LC_ALL, 0)), TRUE);
  }
  Session::set('locale', setlocale(LC_ALL, 0));
}

/// DebugStack::StopTimer('CoreLangLoad');

// Init template engine
esf_Template::getInstance();
if (Registry::get('ClearCache')) esf_Template::getInstance()->Template->ClearCache();

// check if requested module is enabled
if (!ModuleEnabled(Registry::get('esf.Module'))) {
  if (Registry::get('esf.Module') == Registry::get('StartModule')) {
    die('<p>Default module "'.Registry::get('esf.Module').'" is disabled!</p>'
       .'<p>Please <a href="setup/">configure</a> a different default module!</p>');
  }
  Messages::addError(Translation::get('Core.ModuleNotFound', Registry::get('esf.Module')));
  Core::Redirect(Core::URL(array('module'=>Registry::get('StartModule'))));
} elseif (!Core::CheckRequired('module', Registry::get('esf.Module'), $Err)) {
  Messages::addError($Err);
  Core::Redirect(Core::URL(array('module'=>Registry::get('StartModule'))));
}

/// DebugStack::StartTimer('MoreLangLoad', 'Load plugin / module languages');

if (Registry::get('EnglishAsDefault')) {
  // include as default all english texts
  Core::IncludeSpecial(esf_Extensions::$Types, 'language/en');
}

// include all translations
if (!Registry::get('EnglishAsDefault') OR Session::get('Language') != 'en') {
  // include only for enabled modules and plugins
  Core::IncludeSpecial(esf_Extensions::$Types, 'language/'.Session::get('language'));
}

/// DebugStack::StopTimer('MoreLangLoad');

// ----------------------------------------------------------------------------
// pre process
// ----------------------------------------------------------------------------
if (esf_User::isValid()) esf_Auctions::Load();

Event::ProcessInform('PageStart');

$actions = getModuleVar(Registry::get('esf.Module'), 'Actions', FALSE);
if (empty($actions)) $actions = 'index';


/**
 * TODO: >> esfmodule.class.php
 *
$actions = explode('|', $actions);
// supress not supported module actions
if (!in_array(Registry::get('esf.Action'), $actions)) {
  trigger_error(sprintf('Unknown module action: %s/%s!',
                        Registry::get('esf.Module'), Registry::get('esf.Action')));
  Registry::set('esf.Action', $actions[0]);
}
unset($actions);
*/

if (!esf_User::isValid() AND
    Registry::get('Module.'.Registry::get('esf.Module').'.LoginRequired')) {
  /// DebugStack::Info('Forward: '.Registry::get('esf.Module').' -> Login');
  if (Registry::get('esf.Module') != 'login') {
    unset($_REQUEST['ESFSESSID']);
    Session::setP('LoginReturn', $_REQUEST);
  }
  Core::Forward('login');
} elseif (esf_User::isValid() AND $ret = Session::getP('LoginReturn')) {
  Session::setP('LoginReturn');
  Core::Forward(NULL, NULL, $ret);
}

// generic styles/scripts, from defined layout or fallback layout
TplData::add('HtmlHeader.raw', StylesAndScripts('.', Registry::get('Layout')));

if (strtoupper($_SERVER['REQUEST_METHOD']) == 'GET')
  Session::set('returnto', @$_REQUEST['returnto']);

$savemodule = FALSE;

// module preparation
do {

  $title = Translation::getNVL(Registry::get('esf.Module').'.Title',
                               ucwords(Registry::get('esf.Module')));
  TplData::set('Title', $title);
  TplData::set('SubTitle1', $title);
  /// TplData::add('SubTitle1', '<tt style="color:red"> [DEV]</tt>');

  TplData::set('SubTitle2', '');

  // >> Debug
  DebugStack::Info('Processing '.Registry::get('esf.Module').' / '.Registry::get('esf.Action'));
  // << Debug

  if ($savemodule != Registry::get('esf.Module')) unset($Module);

  $savemodule = Registry::get('esf.Module');

  if (!isset($Module)) {
    $file = 'module/'.Registry::get('esf.Module').'/module.class.php';
    if (file_exists($file)) {
      Loader::Load($file);
      $ModuleClass = 'esf_Module_'.Registry::get('esf.Module');
      $Module = new $ModuleClass;
    }
  }
  if (isset($Module)) $Module->handle(Registry::get('esf.Action'));

  // handle ReturnTo=...
  $returnto = decodeReturnTo(Session::get('returnto'));
  if (!empty($returnto) AND
      (strtoupper($_SERVER['REQUEST_METHOD']) == 'POST' OR isset($returnto['force']))) {

    // >> Debug
    //_dbg($returnto, 'ReturnTo');
    // << Debug

    unset($returnto['force']);

    foreach ($returnto as $param => $value) {
      switch ($param) {
        case 'module': Registry::set('esf.Module', $value);  break;
        case 'action': Registry::set('esf.Action', $value);  break;
        default      : $_REQUEST[$param] = $value;           break;
      }
    }
    Session::set('returnto');
  }

  $ModuleLayout = FindActualLayout(Registry::get(esf_Extensions::MODULE.'.'.Registry::get('esf.Module').'.Layout', 'default'));

  // module specific styles/scripts, from defined layout
  TplData::add('HtmlHeader.raw', StylesAndScripts('module/'.Registry::get('esf.Module'), $ModuleLayout));

  // loop until no more forwarded
} while ($savemodule != Registry::get('esf.Module'));
unset($savemodule);

// ----------------------------------------------------------------------------
// process
// ----------------------------------------------------------------------------

Event::ProcessInform('ProcessStart');

if (esf_User::isValid()) {
  // reread auctions AND GROUPS!!
  esf_Auctions::Load();

  if ($last = Event::ProcessReturn('getLastUpdate')) {
    TplData::set('LastUpdate', strftime(Registry::get('Format.DateTimeS'), $last));
    unset($last);
  }
  TplData::set('User', esf_User::getActual());
}

TplData::setConstant('ESF.TITLE', ESF_TITLE);
TplData::setConstant('ESF.SLOGAN', ESF_SLOGAN);
TplData::setConstant('ESF.VERSION', ESF_VERSION);
TplData::setConstant('ESF.RELEASE', ESF_RELEASE);
TplData::setConstant('ESF.HOMEPAGE', ESF_HOMEPAGE);
TplData::setConstant('ESF.AUTHOR', ESF_AUTHOR);
TplData::setConstant('ESF.EMAIL', ESF_EMAIL);
TplData::setConstant('ESF.LONG_TITLE', ESF_LONG_TITLE);
TplData::setConstant('ESF.FULL_VERSION', ESF_FULL_VERSION);
TplData::setConstant('ESF.FULL_TITLE', ESF_FULL_TITLE);
TplData::setConstant('ESF.APPID', APPID);
TplData::setConstant('ESF.LANGUAGE', Session::get('language'));
TplData::setConstant('YEAR', date('Y'));
TplData::setConstant('ESNIPER.VERSION', Session::get('esniperVersion'));
TplData::setConstant('YUELO_VERSION', 'Yuelo - Template engine V. '.Yuelo::VERSION);
TplData::setConstant('PHP.VERSION', PHP_VERSION);

if (stristr($_SERVER['SERVER_SOFTWARE'], 'Apache')) {
  TplData::setConstant('SERVER', array('NAME' => 'apache', 'URL' => 'www.apache.org'));
} elseif (stristr($_SERVER['SERVER_SOFTWARE'], 'lighttpd')) {
  TplData::setConstant('SERVER', array('NAME' => 'lighttpd', 'URL' => 'www.lighttpd.net'));
} elseif (stristr($_SERVER['SERVER_SOFTWARE'], 'IIS')) {
  TplData::setConstant('SERVER', array('NAME' => 'iis', 'URL' => 'www.microsoft.com/iis'));
} else {
  preg_match('~^[\w\s]+~', $_SERVER['SERVER_SOFTWARE'], $args);
  TplData::setConstant('SERVER', array('NAME' => trim($args[0]), 'URL' => NULL));
}

TplData::setConstant('SERVER.VERSION', $_SERVER['SERVER_SOFTWARE']);

TplData::set('Ebay_Homepage', Registry::get('ebay.Homepage'));
TplData::set('Layouts', getLayouts());
TplData::set('GetCategoryFromGroup', FROMGROUP);

TplData::set('FormAction', Core::URL(array('module'=>Registry::get('esf.Module'))));
TplData::set('NoJS', Registry::get('NoJS'));
TplData::set('Layout', Registry::get('LAYOUT'));

$bc = Translation::getNVL(Registry::get('esf.Module').'.TitleIndex', TplData::get('Title'));

// ----------------------------------------------------------------------------
// post process / output
// ----------------------------------------------------------------------------
// load additional layout specific code for all layouts, if exists
$path = 'module/'.Registry::get('esf.Module').'/layout/';
Loader::Load($path . 'layout.php', TRUE, FALSE);
// for actual module layout, if exists
Loader::Load($path . $ModuleLayout.'.php', TRUE, FALSE);

foreach (esf_Extensions::$Types as $ExtType) {
  foreach (esf_Extensions::getExtensions($ExtType) as $Ext) {
    $ExtKey = strtoupper($ExtType).'S.'.strtoupper($Ext);
    $enabled = esf_Extensions::checkState($ExtType, $Ext, esf_Extensions::BIT_ENABLED);
    TplData::setConstant($ExtKey.'.ENABLED', $enabled);
    if ($enabled AND $vars = @$esf_ExtensionVars[$ExtType][$Ext]) {
      foreach ($vars as $key => $val) {
        $key = strtoupper($key);
        if (strpos('|NAME|AUTHOR|EMAIL|VERSION|', "|$key|") === FALSE) {
          TplData::setConstant($ExtKey.'.'.$key, $val);
        }
      }
    }
  }
}

// put actual module configuration in separate (common) array
foreach (getModuleVar(Registry::get('esf.Module')) as $key => $val)
  TplData::setConstant('MODULE.'.strtoupper($key), $val);
unset($ExtType, $Event, $pExt, $vars, $key, $val);

Event::ProcessInform('OutputStart');

/// DebugStack::StartTimer('LoadPluginStyles', 'Load plugin styles and scripts');

// plugin specific styles/scripts, from defined layout or fallback layout
foreach (esf_Extensions::getExtensions(esf_Extensions::PLUGIN) as $plugin) {
  if (PluginEnabled($plugin)) {
    $PluginLayout = FindActualLayout(Registry::get(esf_Extensions::PLUGIN.'.'.$plugin.'.Layout', 'default'));
    TplData::add('HtmlHeader.raw', StylesAndScripts('plugin/'.$plugin, $PluginLayout));
  }
}

/// DebugStack::StopTimer('LoadPluginStyles');
/// DebugStack::StartTimer('BuildMenus', 'Build menus');

// language selectors
foreach ((array)$esf_Languages as $name => $desc) {
  TplData::set('Language.'.$name.'.Name', $name);
  TplData::set('Language.'.$name.'.Desc', $desc);
  TplData::set('Language.'.$name.'.URL',  Core::URL(array('params'=>array('language' => $name))));
}

// build menu
$menustyle = Registry::get('MenuStyle');
if (!is_array($menustyle)) $menustyle = explode(',', $menustyle);

Event::ProcessInform('BuildMenu');

TplData::set('Menu.Main',   esf_Menu::getMain($menustyle[0]));
TplData::set('Menu.Module', esf_Menu::getModule($menustyle[1]));
TplData::set('Menu.System', esf_Menu::getSystem($menustyle[2]));

/// DebugStack::StopTimer('BuildMenus');
/// DebugStack::StartTimer('esniperBugs', 'Check for esniper bug reports');

if (esf_User::isValid()) {
  // check for encountered esniper bug
  $bug = glob(esf_User::UserDir().'/esniper.*.html');
  if (count($bug)) {
    $bugdir = esf_User::UserDir().'/esniper.bug/';
    checkDir($bugdir);
    foreach ($bug as $file) {
      $dest = sprintf('%s%s', $bugdir, basename($file));
      if ($Exec->Move($file, $dest, $res)) {
        Messages::addError($res);
      }
    }
    Messages::addError(Translation::get('Core.EsniperEncounteredBug', $bugdir));
  }
}
unset($bug, $bugdir, $file, $dest, $res, $Error);

/// DebugStack::StopTimer('esniperBugs');
/// DebugStack::StartTimer('HTMLStart', 'HTML start');

$mpath = 'module/'.Registry::get('esf.Module').'/layout/';
$TplDir = array(
  $mpath.$ModuleLayout,
  $mpath.Registry::get('Layout'),
  $mpath.'default',
  'layout/'.Registry::get('Layout'),
  'layout/default',
);

if (!DEVELOP) ob_start();

$esf_Template = esf_Template::getInstance();

Yuelo::set('Language', Session::get('Language'));
Yuelo::set('Layout', $ModuleLayout);
$RootDir = array(
  BASEDIR.'/module/'.Registry::get('esf.Module').'/layout',
  BASEDIR.'/layout',
);

$esf_Template->Output('doctype', FALSE, $RootDir);

$html = $esf_Template->Render('html.start', TRUE, $RootDir);
Event::Process('OutputFilterHtmlStart', $html);
Event::Process('OutputFilter', $html);
echo $html;

$steps = Registry::get('esf.contentonly')
       ? array('content')
       : array('header', 'content', 'footer');

/// DebugStack::StopTimer('HTMLStart');

if (!DEVELOP) ob_end_flush();

/// DebugStack::StartTimer('IndexSteps', 'HTML Content steps');

foreach ($steps as $step) {
  /// DebugStack::StartTimer($step.'block', ucwords($step).' block');

  Event::ProcessInform('Output'.$step);

  if (isset($Module)) $Module->handle(Registry::get('esf.Action'), $step);

  if (!DEVELOP) ob_start();

  if ($step == 'content') {

    TplData::set('esf_Messages', implode((array)Messages::get()));

    if (!Registry::get('esf.contentonly')) {
      $html = $esf_Template->Render('html.messages', TRUE, $RootDir);
      Event::Process('OutputFilter', $html);
      echo $html;
    }

    // if actual module is only a display module, try to render $step.Registry::get('esf.Action') ...
    if (TplData::isEmpty('Content')) {
      $content = $esf_Template->Render('content.'.Registry::get('esf.Action'), FALSE, $RootDir);
      TplData::set('Content', $content);
    }

    // Render general template 'content', if exists
    $content = $esf_Template->Render('content', FALSE, $RootDir);
    if ($content) TplData::set('Content', $content);

  }

  $html = $esf_Template->Render('html.'.$step, TRUE, $RootDir);
  Event::Process('OutputFilter'.$step, $html);
  Event::Process('OutputFilter', $html);
  echo $html;

  if (!DEVELOP) ob_end_flush();

  /// DebugStack::StopTimer($step.'block');
}

/// DebugStack::StopTimer('IndexSteps');

if (!DEVELOP) ob_start();

if ($_DEBUG) {
  $includes = get_included_files();
  DebugStack::Debug('Total included files: '.count($includes));
  if ($_TRACE) DebugStack::Save($_TRACE);
  $msgs = DebugStack::Render(TRUE);
  echo <<<EOT
<div class="dbgtitle"
     style="margin-top:1em;cursor:pointer;text-align:center;padding:5px"
     onclick="$('dbgmsgs').toggle()"><tt><strong>DEBUG</strong></tt></div>
<div id="dbgmsgs" style="height:40em;overflow:auto;display:none">$msgs</div>
EOT;
}
// << Debug

$html = $esf_Template->Render('html.end', TRUE, $RootDir);
Event::Process('OutputFilterHtmlEnd', $html);
Event::Process('OutputFilter', $html);
echo $html;

Event::ProcessInform('PageEnded');

Session::close();

Yuelo_Cache::gc();