<?php
/**
 * @category   Plugin
 * @package    Plugin-UrlRewriteHash
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    0.1.0
 */

/**
 * Rewrite urls
 *
 * @category   Plugin
 * @package    Plugin-UrlRewriteHash
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    Release: @package_version@
 */
class esf_Plugin_UrlRewriteHash extends esf_Plugin {

  /**
   * URL Parameter
   */
  const PARAM = 'go';

  /**
   * Parameter delimiter
   * @ignore
   */
  const DELIM = "\x01";

  /**
   * URL rewrite function
   *
   * @param array &$args URL parameters
   * @return string URL
   */
  public function UrlRewriteNotification( &$args ) {
    # return, module, action, params, anchor
    $url = $module = $action = $anchor = NULL;
    $params = array();
    @list($url, $module, $action, $params, $anchor) = $args;

    if (empty($url)) {
      $url = $_SERVER['PHP_SELF'];
      $params['module'] = $module;
      $params['action'] = $action;
      $p = array();
      foreach ($params as $name => $val) {
        if (is_array($val)) continue;
        if (is_numeric($name)) {
          $name = $val;
          $val = FALSE;
        }
        $pp = $name;
        if ($val !== '') {
          $pp .= '='.urlencode($val);
        }
        $p[] = $pp;
      }
      $url .= '?' . self::PARAM.trim(base64_encode(implode(self::DELIM,$p)), '=');
      if ($anchor) {
        $url .= '#' . $anchor;
      }
    }
    $args[0] = $url;
  }

  /**
   * URL un-rewrite function
   *
   * @param array &$args Request parameters
   */
  public function UrlUnRewriteNotification( &$args ) {
    $hlen = strlen(self::PARAM);

    foreach (array_keys($args) as $param) {
      if (substr($param, 0, $hlen) === self::PARAM) {
        $p = base64_decode(substr($param, $hlen));
        unset($args[$param]);
        $pp = explode(self::DELIM, $p);
        foreach ($pp as $p) {
          $p = explode('=', $p, 2);
          $args[$p[0]] = isset($p[1]) ? $p[1] : '';
        }
        break;
      }
    }
  }

}

Event::attach(new esf_Plugin_UrlRewriteHash);