#! /bin/bash
# ----------------------------------------------------------------------------
# /var/install/config.d/esniper-update.sh - paramater update script
#
# Creation   : 2010-05-20 dr_snuggles
# Last update: $Id: esniper-update.sh 24179 2010-05-23 12:29:50Z dr_snuggles $
#
# Copyright (c) 2010 the eisfair team, <team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/esniper-update-trace$$.log
#set -x

# include configlib for using printvar
. /var/install/include/configlib

# include eislib
. /var/install/include/eislib

# include apache2 config
. /etc/config.d/apache2

# set packages name
packageName=esniper

# ----------------------------------------------------------------------------
# Set the default values for configuration
# ----------------------------------------------------------------------------
START_ESNIPER='no'
ESNIPER_HTML_FOLDER="${APACHE2_DOCUMENT_ROOT}/esniper"


# ----------------------------------------------------------------------------
# Read old configuration and rename old variables
# ----------------------------------------------------------------------------
renameOldVariables()
{
    # read old values
    if [ -f /etc/config.d/${packageName} ]
    then
        . /etc/config.d/${packageName}
    fi
}

# ----------------------------------------------------------------------------
# Write config and default files
# ----------------------------------------------------------------------------
makeConfigFile()
{
    internal_conf_file=${1}
    {
    # ------------------------------------------------------------------------
    printgpl -conf ${packageName} '2010-05-20' 'dr_snuggles'
    # ------------------------------------------------------------------------


    # ------------------------------------------------------------------------
    printgroup "Basic configuration"
    # ------------------------------------------------------------------------
    printvar 'START_ESNIPER' 'Use: yes or no'
    printvar "ESNIPER_HTML_FOLDER" "html folder for esniper frontend"

    # ------------------------------------------------------------------------
    printend
    # ------------------------------------------------------------------------

    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

# ----------------------------------------------------------------------------
# Create the check.d file
# ----------------------------------------------------------------------------
makeCheckFile()
{
    printgpl -check ${packageName} '2010-05-20' 'dr_snuggles' >/etc/check.d/${packageName}
    cat >> /etc/check.d/${packageName} <<EOFG
# Variable            OPT_VARIABLE      VARIABLE_N           VALUE
START_ESNIPER         -                 -                    YESNO
ESNIPER_HTML_FOLDER   START_ESNIPER     -                    ABS_PATH

EOFG

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${packageName}
    chown root /etc/check.d/${packageName}

#    printgpl -check_exp ${packageName} '2010-05-20' 'dr_snuggles' >/etc/check.d/${packageName}.exp
#    cat >> /etc/check.d/${packageName}.exp <<EOFG
#
#
#EOFG

    # Set rights for check.exp file
#    chmod 0600 /etc/check.d/${packageName}.exp
#    chown root /etc/check.d/${packageName}.exp

#    printgpl -check_ext ${packageName} '2010-05-20' 'dr_snuggles' >/etc/check.d/${packageName}.ext
#    cat >> /etc/check.d/${packageName}.ext <<EOFG
#
#
#EOFG

    # Set rights for check.ext file
#    chmod 0600 /etc/check.d/${packageName}.ext
#    chown root /etc/check.d/${packageName}.ext
}



# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------
# Write default config file
if [ -f /etc/config.d/${packageName} ]
then
    mecho -info -n 'Updating configuration.'
else
    mecho -info -n 'Creating configuration.'
fi

makeConfigFile /etc/default.d/${packageName}

# Update from old version
mecho -info -n '.'
renameOldVariables

# Write new config file
mecho -info -n '.'
makeConfigFile /etc/config.d/${packageName}

# Write check.d file
mecho -info -n '.'
makeCheckFile

mecho -info ' Finished.'

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------